# Licensed Materials - Property of IBM
# 5725-C94, 5725-C95, 5725-C96
# Copyright IBM Corporation 2012. All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.

import sys
import getopt
from java.util import ResourceBundle
from java.text import MessageFormat;

def usage():
	print
	print "Usage:"
	print
	print "   wsadmin.(sh|bat) -f registerBusinessMonitorTarget.py [OPTIONS]"
	print
	print "Options:"
	print "   -d, --deployment-environment=DE_NAME     Specify the deployment environment name."
	print "   -h, --host=host_name                     Business Monitor deployment manager host name."
	print "   -p, --port=port                          Business Monitor deployment manager SOAP port."
	print "   -a, --auth-alias-name=auth_alias_name    J2C authentication alias with Business Monitor"
	print "                                            administrative user credentials."
	print "   -k, --dashboard-kpi-administrator=userid User id for generated Business Monitor"
	print "                                            dashboard's KPI-Administrator role.."
	print "       --no-sync                            Do not force the immediate propagation"
	print "                                            of changes across nodes."
	print "       --no-save                            Do not save configuration changes."
	print "       --help                               Display this help"
	print


def main():
	try:
		options, args = getopt.getopt(sys.argv, "d:h:p:a:k:", ["no-save", "no-sync", "deployment-environment=", "host=", "port=", "auth-alias-name=", "dashboard-kpi-administrator="])
	except getopt.GetoptError, err:
		print str(err)
		usage()
		return 2

	save = 1
	sync = 1
	deploymentEnvironmentName = None
	businessMonitorTarget = {'type':'SOAP','host':None, 'port':None, 'authenticationAliasName':None}
	kpiAdministrator = None
	
	for option, arg in options:
		if option in ("-d", "--deployment-environment"):
			deploymentEnvironmentName = arg
		elif option in ("-h", "--host"):
			businessMonitorTarget["host"] = arg
		elif option in ("-p", "--port"):
			businessMonitorTarget["port"] = arg
		elif option in ("a", "--auth-alias-name"):
			businessMonitorTarget["authenticationAliasName"] = arg
		elif option in ("k", "--dashboard-kpi-administrator"):
			kpiAdministrator = arg
		elif option == "--no-save":
			save = 0
			sync = 0
		elif option == "--no-sync":
			sync = 0
		elif option == "--help":
			usage()
			return
		else:
			assert 0, "un-handled option"

	# host parameter must be specified
	if not businessMonitorTarget["host"]:
		print getMessage('BPM_BUSINESS_MONITOR_TARGET_CFG_MISSING_PARAM', 'The %s parameter must be specified.', 'host');
		usage()
		return 2
	
	# port parameter must be specified
	if not businessMonitorTarget["port"]:
		print getMessage('BPM_BUSINESS_MONITOR_TARGET_CFG_MISSING_PARAM', 'The %s parameter must be specified.', 'port');
		usage()
		return 2
	
	# port parameter must be specified
	if not businessMonitorTarget["authenticationAliasName"]:
		print getMessage('BPM_BUSINESS_MONITOR_TARGET_CFG_MISSING_PARAM', 'The %s parameter must be specified.', 'auth-alias-name');
		usage()
		return 2
	
	# if specified, the deployment environment should exist
	deploymentEnvironment = None
	if deploymentEnvironmentName:
		deploymentEnvironment = AdminConfig.getid('/BPMDeploymentEnvironment:%s/' % deploymentEnvironmentName)
		if not deploymentEnvironment:
			raise getMessage('BPM_BUSINESS_MONITOR_TARGET_CFG_INVALID_DE', 'Deployment environment "%s" does not exist.', deploymentEnvironmentName)
	
	# is specified, the J2C authentication alias should exist
	targetAuthenticationAlias = None
	if businessMonitorTarget["authenticationAliasName"]:
		authenticationAliases = AdminConfig.list('JAASAuthData').splitlines()
		for possibleAuthenticationAlias in authenticationAliases:
			if AdminConfig.showAttribute(possibleAuthenticationAlias, 'alias') == businessMonitorTarget['authenticationAliasName']:
				targetAuthenticationAlias = possibleAuthenticationAlias
				break
		if not targetAuthenticationAlias:
			raise getMessage('BPM_BUSINESS_MONITOR_TARGET_CFG_INVALID_AUTH_ALIAS', 'J2C authentication alias "%s" does not exist.', businessMonitorTarget['authenticationAliasName'])
		
	# update the default business monitor target configuration
	if deploymentEnvironment:
		registerBusinessMonitorTarget(deploymentEnvironment, businessMonitorTarget, kpiAdministrator)
	else:
		registerBusinessMonitorTargets(businessMonitorTarget, kpiAdministrator)
	
	# save and sync
	if save:
		print getMessage('BPM_PROXY_CFG_CONFIG_SAVING', "Configuration is being saved.", ())
		AdminConfig.save()
		if sync:
			print getMessage('BPM_PROXY_CFG_SYNCHRONIZING', "Synchronizing the configuration across all nodes.", ())
			AdminNodeManagement.syncActiveNodes()
	
	print getMessage('BPM_PROXY_CFG_DONE', "Done.", ())

def registerBusinessMonitorTargets(businessMonitorTargetAttributes, kpiAdministrator):
	deploymentEnvironments = AdminConfig.list('BPMDeploymentEnvironment').splitlines()
	print 
	for deploymentEnvironment in deploymentEnvironments:
		registerBusinessMonitorTarget(deploymentEnvironment, businessMonitorTargetAttributes, kpiAdministrator)
	
		
def registerBusinessMonitorTarget(deploymentEnvironment, businessMonitorTargetAttributes, kpiAdministrator):
	# get the 'default' business monitor target
	businessMonitorTarget = None
	if AdminConfig.showAttribute(deploymentEnvironment, 'businessMonitorTarget'):
		businessMonitorTargets = AdminUtilities.convertToList(AdminConfig.showAttribute(deploymentEnvironment, 'businessMonitorTarget'))
		for possibleBusinessMonitorTarget in businessMonitorTargets:
			if AdminConfig.showAttribute(possibleBusinessMonitorTarget, 'name') == 'default':
				businessMonitorTarget = possibleBusinessMonitorTarget
				print getMessage('BPM_BUSINESS_MONITOR_TARGET_CFG_FOUND_MON_CFG', "Found %s business monitor target configuration", 'default')
				break
	
	# if none found, create a new one, set name to 'default'
	if not businessMonitorTarget:
		attributes = []
		attributes.append(['name', 'default'])
		businessMonitorTarget = AdminConfig.create('BPMBusinessMonitorTarget',deploymentEnvironment,attributes,'businessMonitorTarget')
		print getMessage('BPM_BUSINESS_MONITOR_TARGET_CFG_CREATED_MON_CFG', 'Business Monitor target configuration "%s" created.' , 'default')
	
	# get the was admin connection info
	wasAdminConnection = AdminConfig.showAttribute(businessMonitorTarget, 'wasAdminConnection')
	
	# if none found, create a new one
	if not wasAdminConnection:
		print getMessage('BPM_BUSINESS_MONITOR_TARGET_CFG_CREATED_MON_CONNECTION_CFG', 'Connection configuration for Business Monitor target ''%s'' created.', 'default')
		wasAdminConnection = AdminConfig.create('BPMWasAdminConnection', businessMonitorTarget, [], 'wasAdminConnection')
	
	# update type, host, port, authentication alias
	attributes = []
	attributes.append(['type', businessMonitorTargetAttributes['type']])
	attributes.append(['host', businessMonitorTargetAttributes['host']])
	attributes.append(['port', businessMonitorTargetAttributes['port']])
	attributes.append(['authenticationAliasName', businessMonitorTargetAttributes['authenticationAliasName']])
	AdminConfig.modify(wasAdminConnection, attributes)
	print getMessage('BPM_BUSINESS_MONITOR_TARGET_CFG_CREATED_MON_CONNECTION_SET_TYPE', 'Set Business Monitor connection type to %s', businessMonitorTargetAttributes['type'])
	print getMessage('BPM_BUSINESS_MONITOR_TARGET_CFG_CREATED_MON_CONNECTION_SET_HOST', 'Set Business Monitor connection host name to %s', businessMonitorTargetAttributes['host'])
	print getMessage('BPM_BUSINESS_MONITOR_TARGET_CFG_CREATED_MON_CONNECTION_SET_PORT', 'Set Business Monitor connection port to %s', businessMonitorTargetAttributes['port'])
	print getMessage('BPM_BUSINESS_MONITOR_TARGET_CFG_CREATED_MON_CONNECTION_SET_AUTH_ALIAS', 'Set Business Monitor connection authentication alias name to %s', businessMonitorTargetAttributes['authenticationAliasName'])
	
	# get the dashboard data security for resource group 'root'
	dashboardDataSecurity = AdminConfig.showAttribute(businessMonitorTarget, 'dashboardDataSecurity')
	
	# if not found, create dashboard data security for resource group 'root'
	if not dashboardDataSecurity:
		dashboardDataSecurity = AdminConfig.create('BPMDashboardDataSecurity', businessMonitorTarget, [['resourceGroup', 'root']], 'dashboardDataSecurity')
		print getMessage('BPM_BUSINESS_MONITOR_TARGET_CFG_CREATED_MON_BSPACE_DATA_SECURITY_CFG', 'Dashboard data security configuration for resource group "%s" created.', 'root')
	
	# find the user role mapping for KPIAdministrator role
	kpiAdministratorUserRoleMapping = None
	userRoleMappingsAsString = AdminConfig.showAttribute(dashboardDataSecurity, 'userRoleMapping') 
	if userRoleMappingsAsString:
		userRoleMappings = AdminUtilities.convertToList(userRoleMappingsAsString)
		for userRoleMapping in userRoleMappings:
			if AdminConfig.showAttribute(userRoleMapping, 'role') == 'KPI-Administrator':
				kpiAdministratorUserRoleMapping = userRoleMapping
				break
	
	# if not found create the user role mapping for KPI-Administrator
	if not kpiAdministratorUserRoleMapping:
		kpiAdministratorUserRoleMapping = AdminConfig.create('BPMUserRoleMapping', dashboardDataSecurity, [['role', 'KPI-Administrator']], 'userRoleMapping')
		print getMessage('BPM_BUSINESS_MONITOR_TARGET_CFG_CREATED_MON_BSPACE_DATA_SECURITY_USER_ROLE_MAPPING_CFG', 'User to role mapping configuration for resource group "%s" created.', 'root')
	
	# update kpi administrator user id if specified
	if kpiAdministrator:
		AdminConfig.modify(kpiAdministratorUserRoleMapping, [['userId', kpiAdministrator]])
		print getMessage('BPM_BUSINESS_MONITOR_TARGET_CFG_MON_BSPACE_DATA_SECURITY_KPI_ADMIN_ROLE_MAPPING', 'User id ''%s'' mapped to the ''%s'' dashboard data security role.', (kpiAdministrator, 'KPI-Administrator'))
		
	# if not already set, use the user from the authentication alias
	if not AdminConfig.showAttribute(kpiAdministratorUserRoleMapping, 'userId'):
		for authenticationAlias in AdminConfig.list('JAASAuthData').splitlines():
			if AdminConfig.showAttribute(authenticationAlias, 'alias') == businessMonitorTargetAttributes['authenticationAliasName']:
				userId = AdminConfig.showAttribute(authenticationAlias, 'userId')
				AdminConfig.modify(kpiAdministratorUserRoleMapping, [['userId',userId]])
				print getMessage('BPM_BUSINESS_MONITOR_TARGET_CFG_MON_BSPACE_DATA_SECURITY_KPI_ADMIN_ROLE_MAPPING', 'User id ''%s'' mapped to the ''%s'' dashboard data security role.', (userId, 'KPIAdministrator'))
				break

def getMessage(key, default, args=()):
	# Allows using internationalized strings, or default embedded
	# if copied to another install
	try:
		bundle = ResourceBundle.getBundle("WLEPIIMessages")
		msg = bundle.getString(key)
		if msg is None:
			return strIfNotFound % args
		else:
			return MessageFormat.format(msg, args)
	except:
		# tried to get the bundle and failed, so return the string
		return default % args

import linecache
def traceline(frame, event, arg):
    if event == "line":
		lineNumber = frame.f_lineno
		line = linecache.getline("registerBusinessMonitorTarget.py", lineNumber)
		print "%d: %s" % (lineNumber, line.strip())
    return traceline
			
if __name__ == "__main__":
	# sys.settrace(traceline)
	sys.exit(main());

